@@ -7,7 +7,7 @@ import json
 import pluggy
 from hash import resource_hookimpl as hookimpl
 
-from hash import errors, utils, hash
+from hash import errors, utils, core
 
 from .targets import Target, get_target
 all_actions = ["build", "test", "publish", "deploy"]
@@ -740,15 +740,21 @@ class DockerImageResource(Resource):
         save_command = f"docker save {image_name} -o docker.build"
         try:
             self.execute(save_command, self.getPath())
-            ret["artifacts"] = [
-                self.create_artifact("image_file", "build", env.getName(
-                ), state["hash"], "file", os.path.join(self.getPath(), "docker.build"))
-            ]
+            if env:
+                ret["artifacts"] = [
+                    self.create_artifact("image_file", "build", env.getName(
+                    ), state["hash"], "file", os.path.join(self.getPath(), "docker.build"))
+                ]
+            else:
+                ret["artifacts"] = [
+                    self.create_artifact("image_file", "build", None, state["hash"], "file", os.path.join(
+                        self.getPath(), "docker.build"))
+                ]
         except subprocess.CalledProcessError as e:
             raise errors.ResourceBuildError(e)
         return ret
 
-    @classmethod
+    @ classmethod
     def runTests(cls, image_name, path, accept_latest=False, docker_file="Dockerfile"):
         test_command = f"docker build -t {image_name} . -f {docker_file}"
         try:
@@ -825,7 +831,7 @@ class DockerImageResource(Resource):
 
 
 class GoServiceResource(Resource):
-    @hookimpl
+    @ hookimpl
     def init(self, file: str):
         """
             Initialize the GoService resource.
@@ -839,7 +845,7 @@ class GoServiceResource(Resource):
                 "This is not a GoService resource")
         self.name = self.getMetadata("name")
 
-    @classmethod
+    @ classmethod
     def compile(cls, go_executable, executable_name, path):
         go_version_command = f"{go_executable} version"
         ret = {}
@@ -856,7 +862,7 @@ class GoServiceResource(Resource):
         except subprocess.CalledProcessError as e:
             raise errors.ResourceBuildError(e)
 
-    @classmethod
+    @ classmethod
     def runTests(cls, go_executable, golangci_lint_executable, path):
         golangci_lint_version_command = f"{golangci_lint_executable} version"
         ret = {}
@@ -875,7 +881,7 @@ class GoServiceResource(Resource):
         except subprocess.CalledProcessError as e:
             raise errors.ResourceBuildError(e.output.decode("utf-8"))
 
-    @hookimpl
+    @ hookimpl
     def action(self, name, state, env):
         if name == "build":
             return self.build(state, env)
@@ -898,13 +904,13 @@ class GoServiceResource(Resource):
             "status": "Go service tested without errors"
         }
 
-    @hookimpl
+    @ hookimpl
     def re_action(state: dict, action: str, env={}) -> bool:
         return False
 
 
 class MicroService(object):
-    @classmethod
+    @ classmethod
     def build(cls, res, state, target):
         docker_path = os.path.join(
             res.getPath(), res.getSpec("docker_path", "."))
@@ -941,7 +947,7 @@ class MicroService(object):
                 f"Cannot replace IMAGE URL in manifests file : error {p.stderr.decode('utf-8')}")
         return image_url, k8s_file
 
-    @classmethod
+    @ classmethod
     def test(cls, res, target, artifacts):
         # Test docker image
         image_name = res.getSpec("image_name", res.name)
@@ -955,7 +961,7 @@ class MicroService(object):
                 KustomizeResource.runTests(target, k8s_path, [artifact])
                 break
 
-    @classmethod
+    @ classmethod
     def deploy(cls, res, target, artifacts):
         config = {
             "path": res.getPath()
@@ -969,7 +975,7 @@ class MicroService(object):
 
 # Go Micro Service resource
 class GoMicroServiceResource(Resource):
-    @hookimpl
+    @ hookimpl
     def init(self, file: str):
         """
             Initialize the GoMicroService resource.
@@ -1107,7 +1113,7 @@ class GoMicroServiceResource(Resource):
             except subprocess.CalledProcessError as e:
                 raise errors.ActionError(e)
 
-    @hookimpl
+    @ hookimpl
     def action(self, name, state, env):
         self.setup_tools()
         if name == "build":
@@ -1173,7 +1179,7 @@ class GoMicroServiceResource(Resource):
         artifacts = state["envs"][env.getName()]["build"].get("artifacts", [])
         MicroService.deploy(self, target, artifacts)
 
-    @hookimpl
+    @ hookimpl
     def re_action(self, action, state, env):
         if env is None:
             result = state.get("no_env")
@@ -1229,8 +1235,9 @@ def get_resource(file: str):
 
 
 class ResourceSpace(object):
-    def __init__(self, base: str) -> None:
+    def __init__(self, base: str, store=None) -> None:
         self.__base = base
+        self.__store = store
         hash_dir = os.path.join(base, ".hash")
         if not os.path.isdir(hash_dir):
             os.mkdir(hash_dir)
@@ -1311,10 +1318,22 @@ class ResourceSpace(object):
         resource.fill_specs(env, _globals, self)
 
     def calculate_hash(self, resource, alg="sha256"):
+        ignore = []
+        if self.__store is not None:
+            artifact_paths = self.__store.get_artifact_paths(resource.getId())
+            artifacts_paths = []
+            for path in artifact_paths:
+                new_path = path.replace(resource.getPath(), "")
+                artifacts_paths.append(new_path.lstrip("./"))
+            ignore = artifacts_paths
+        r = core.templates.Renderer(resource, self, [], {})
+        templates = r.get_hash_templates()
+        for template in templates:
+            ignore.append(template.lstrip('./').rstrip(".hash"))
         if resource.getKind() == "Env":
             match = [resource.getFile().split("/")[-1]]
             match.extend(resource.getSpec("match", []))
-            return hash.Hash(alg).hash(os.path.join(self.__base, resource.getPath()), match)
+            return core.Hash(alg).hash(os.path.join(self.__base, resource.getPath()), match)
         match = resource.getSpec("match")
         if match:
             match.append(resource.getFile().split("/")[-1])
@@ -1325,4 +1344,4 @@ class ResourceSpace(object):
         if resource.getKind() == "Env":
             match = [resource.getFile().split("/")[-1]]
             match.extend(resource.getSpec("match", []))
-        return hash.Hash(alg).hash(os.path.join(self.__base, resource.getPath()), match)
+        return core.Hash(alg).hash(os.path.join(self.__base, resource.getPath()), match, ignore)
