@@ -134,23 +134,22 @@ class HashStore(object):
                                 saved=True
                             )
                             f.write(self.read_artifact(aft, resource_id))
-        no_env_artifacts = state.get("no_env", {}).get("artifacts", [])
-        for artf in no_env_artifacts:
-            for ac in core.all_actions:
-                artfs = no_env_artifacts.get(ac, {}).get("artifacts", [])
-                for art in artfs:
-                    if art.get("kind") == "file":
-                        with open(art.get("data"), "wb") as f:
-                            aft = utils.Artifact(
-                                art.get("id"),
-                                art.get("action"),
-                                art.get("env"),
-                                art.get("hash"),
-                                art.get("kind"),
-                                art.get("data"),
-                                saved=True
-                            )
-                            f.write(self.read_artifact(aft, resource_id))
+        for ac in core.all_actions:
+            artfs = state.get("no_env", {}).get(
+                ac, {}).get("artifacts", [])
+            for art in artfs:
+                if art.get("kind") == "file":
+                    with open(art.get("data"), "wb") as f:
+                        aft = utils.Artifact(
+                            art.get("id"),
+                            art.get("action"),
+                            art.get("env"),
+                            art.get("hash"),
+                            art.get("kind"),
+                            art.get("data"),
+                            saved=True
+                        )
+                        f.write(self.read_artifact(aft, resource_id))
 
     def get(self, resource_id: str, h: str):
         """
@@ -172,7 +171,6 @@ class HashStore(object):
             if state is not None and type(state) != State:
                 state.get("hash")
         except AttributeError:
-            print(state)
             raise errors.StateStoreError(
                 f"Invalid state read from state store for resource {resource_id}, hash {h}")
         if state is not None:
@@ -192,6 +190,9 @@ class HashStore(object):
                 f"Got unkonwn state from state store for resource {resource_id} and hash {h}")
         return st
 
+    def get_artifact_paths(self, resource_id):
+        return self.artifact_paths(resource_id)
+
     def store(self, state: State):
         """
         This method is used to store a state in the backend.
@@ -305,13 +306,31 @@ class HashStoreLocalFile(HashStore):
             os.makedirs(path)
         except FileExistsError:
             pass
-        path = os.path.join(path, artifact.file_name())
+        artifact_path = os.path.join(path, artifact.file_name())
         with open(artifact.getData(), "rb") as f:
             file_data = f.read()
-        with open(path, "wb") as f:
+        with open(artifact_path, "wb") as f:
             f.write(file_data)
+        artifacts_path = os.path.join(path, "artifacts.path")
+        paths = []
+        if os.path.isfile(artifacts_path):
+            with open(artifacts_path, "r") as f:
+                paths = f.readlines()
+        if artifact.getRelativePath() not in [x.strip() for x in paths]:
+            with open(artifacts_path, "a") as f:
+                f.write(artifact.getRelativePath())
+                f.write('\n')
         return path
 
+    @store_hookimpl
+    def artifact_paths(self, resource_id):
+        path = os.path.join(self.__path, resource_id.replace(
+            ":", "/"), "artifacts.path")
+        if os.path.isfile(path):
+            with open(path, "r") as f:
+                return f.readlines()
+        return []
+
     @store_hookimpl
     def get_states(self, resource_id):
         path = os.path.join(self.__path, resource_id.replace(":", "/"))
@@ -403,6 +422,37 @@ class HashStoreDOSpace(HashStore):
                                 Body=data,
                                 ACL="private"
                                 )
+        artifacts_paths = f"{self.get_organization()}-{self.get_project()}-{resource_id}-artifacts.paths"
+        try:
+            data = self._client.get_object(Bucket=self._space_name,
+                                           Key=artifacts_paths
+                                           )
+            paths = json.loads(data["Body"].read())
+        except ClientError as e:
+            if e.response["Error"]["Code"] != "NoSuchKey":
+                raise errors.StoreError(e)
+            paths = []
+        if artifact.getRelativePath() not in paths:
+            paths.append(artifact.getRelativePath())
+            self._client.put_object(Bucket=self._space_name,
+                                    Key=artifacts_paths,
+                                    Body=paths,
+                                    ACL="private"
+                                    )
+
+    @store_hookimpl
+    def artifact_paths(self, resource_id):
+        artifacts_paths = f"{self.get_organization()}-{self.get_project()}-{resource_id}-artifacts.paths"
+        try:
+            data = self._client.get_object(Bucket=self._space_name,
+                                           Key=artifacts_paths
+                                           )
+            paths = json.loads(data["Body"].read())
+        except ClientError as e:
+            if e.response["Error"]["Code"] != "NoSuchKey":
+                raise errors.StoreError(e)
+            paths = []
+        return paths
 
     @store_hookimpl
     def get_states(self, uid: str):
